#include <asm/io.h>
#include "lm78.h"

/* June 21, 1998  Philip Edelbrock:  A couple modifications,

	- A test was added to make sure the device isn't busy for some reason, and

	- The start-up code was altered according to my interpretation of the docs (my machine
wasn't refreshing the stats otherwise)

	- Added normalizing constants to adjust measured voltages to the voltages which they represent (i.e., we're compensating for the voltage
divider networks on the mainboard).  These are for P6xxx Supermicro mainboards (as tested on a P6SLS and a P6DBE).

TODO:
	- "Lock" the device so we don't get two instances running at the same time.

	- LM75 stats. (I2C serial bus testing and interface.)

	- Watchdogs and limits control.
*/

/* Constants for Supermicro motherboards */
#define NORMAL_0 1
#define NORMAL_1 1
#define NORMAL_2 1

/* These are ratios suggested by the National Semi docs. */
#define NORMAL_3 1.6778523
/* Suggested +5V ratio */
#define NORMAL_4 4.0
/* Suggested +12V ratio */
#define NORMAL_5 4.0
/* Suggested -12V ratio */
#define NORMAL_6 1.6666667
/* Suggested -5V ratio */



/* Test_Busy:

Docs note that there is a possibility of the LM78 being busy with serial communication.  Before
altering anything, we should read the Address Register and check the highest order bit (used to
indicate that the device is busy).  */

int Test_Busy() {
  if (inb_p(LM78_ADDRESS_REG) & 0x0080) {
	printf("LM78: Busy!");
	return(1);
 }
 return(0);
}


/* main:

	This is where it all happens.  */

int main(int argc, char *argv[]) {
  int fan,i;
  
  ioperm(LM78_BASE,LM78_EXTENT,1);
  ioperm(0x80,1,1);

/* A busy event is rare enough that we can simple bail out if it is detected. */
  if (Test_Busy()) { return(1); }

  /* Reset all except Watchdog values and last conversion values
     This sets fan-divs to 2, and disables any NMI/IRQ or SMI interrupts */
 LM78_WRITE(LM78_CONFIG, 0x80 );

  /* Start monitoring */
  LM78_WRITE(LM78_CONFIG, (LM78_READ(LM78_CONFIG) & 0xf7) | 0x01);

  /* It can take upto 1.5 secs before it has scanned everything */
  sleep(2);

/* A busy event is rare enough that we can simple bail out if it is detected. */
  if (Test_Busy()) { return(1); }

  printf("config: 0x%02X\n", LM78_READ( 0x40 ) );
  printf("mask1: 0x%02X\n", LM78_READ( 0x43 ) );
  printf("mask2: 0x%02X\n", LM78_READ( 0x44 ) );
  printf("nmi mask1: 0x%02X\n", LM78_READ( 0x45 ) );
  printf("nmi mask2: 0x%02X\n", LM78_READ( 0x46 ) );

  /* Ugly code, nice presentation */
  for (i=0; i<= 0x3f; i++) {
    if ((i & 0x0F)  == 0)
      printf("\nPost Ram/Value Ram(0x%02X):",i);
    printf(" %02X", LM78_READ( i ) );
  }
  printf("\n");

  printf("VID/Fan Divisor Register: 0x%02X\n", LM78_READ( LM78_FANDIV ) );
  printf("Serial bus address: 0x%02X\n", LM78_READ( 0x48 ) );

  printf("IN0: %d (%f)\n", LM78_READ(LM78_IN0), LM78_READ(LM78_IN0) * 0.016 * NORMAL_0);
  printf("IN1: %d (%f)\n", LM78_READ(LM78_IN1), LM78_READ(LM78_IN1) * 0.016 * NORMAL_1);
  printf("IN2: %d (%f)\n", LM78_READ(LM78_IN2), LM78_READ(LM78_IN2) * 0.016 * NORMAL_2);
  printf("IN3: %d (%f)\n", LM78_READ(LM78_IN3), LM78_READ(LM78_IN3) * 0.016 * NORMAL_3);
  printf("IN4: %d (%f)\n", LM78_READ(LM78_IN4), LM78_READ(LM78_IN4) * 0.016 * NORMAL_4);
  printf("-IN5: %d (%f)\n", LM78_READ(LM78_MINUS_IN5), -LM78_READ(LM78_MINUS_IN5) * 0.016 * NORMAL_5);
  printf("-IN6: %d (%f)\n", LM78_READ(LM78_MINUS_IN6), -LM78_READ(LM78_MINUS_IN6) * 0.016 * NORMAL_6);

  printf("----------\n");
  printf("Mb_temp: %d\n", LM78_READ( LM78_TEMPERATURE ) );

  fan = LM78_READ( LM78_FAN1 );
  printf("Fan1: ");
  if (fan==255)
    printf( "0 rpm\n" );
  else
    printf("%d rpm\n", LM78_RPM(fan,2) );

  fan = LM78_READ( LM78_FAN2 );
  printf("Fan2: ");
  if (fan==255)
    printf( "0 rpm\n" );
  else
    printf("%d rpm\n", LM78_RPM(fan,2) );

  fan = LM78_READ( LM78_FAN3 );
  printf("Fan3: ");
  if (fan==255)
    printf( "0 rpm\n" );
  else
    printf("%d rpm\n", LM78_RPM(fan,2) );

}

